/*
btsweep.c Version 0.1.0  Btree sweep database
Copyright (C) 2003  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* read the database in physical sequence */
/* one record at a time maintaining position */
/* in bthdr */
/* bthdr->ofst   points to the block */
/* bthdr->rcdnum is the record# in the block */
/* bthdr->rcd    points to the record */

/* valid values for position: */
/* SWPFRST first physical record */
/* SWPNEXT next  physical record */

#include <bthash.h>

int btsweep(btreefmt *bthdr, int position,
   unsigned char *buf)
   {
   BOOL   bResult;
   DWORD  rdlen;
   DWORD  loc;
   DWORD  msgerr;
   LPVOID lpMsgBuf;
   unsigned char *p,*q,*r;
   bthdr->reason = 0;
   if (position == SWPNEXT);
   else if (position == SWPFRST)
      {
      bthdr->seq = 0;
      loc = btseek(bthdr,ROOT);
      if (bthdr->reason) return(loc);
      } /* if first read */
   else
      {
      fprintf(stderr,"btsweep: invalid "
	 "position %d\n", position);
      bthdr->reason = BTREE_ERR + 2301;
      return(BTREE_ERR + 2301);
      } /* if first read */
   /* bypass deleted index records */
   while (1)
      {
      if (position == SWPFRST
         || bthdr->rcdnum >= bthdr->blk->count)
         {
         bResult = ReadFile(bthdr->hndl,
            (unsigned char *) bthdr->blk,
            (DWORD) bthdr->blksz,
            &rdlen,NULL);
         if (!bResult)
            {
            msgerr = GetLastError();
            bthdr->reason = SYS_ERR + msgerr;
            FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
               FORMAT_MESSAGE_FROM_SYSTEM |
               FORMAT_MESSAGE_IGNORE_INSERTS,
               NULL,             /* source */
               msgerr,
               MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
               (LPTSTR) &lpMsgBuf,
               0,                /* message length */
               NULL);            /* insertion values */
            fprintf(stderr,"btsweep: %s", lpMsgBuf);
            LocalFree(lpMsgBuf);
	    fprintf(stderr,"btsweep: %s: read error\n",
	       bthdr->dbname);
            return(SYS_ERR + msgerr);
            } /* if read err */
         else if (rdlen == (DWORD) bthdr->blksz)
            {
	    bthdr->rcdnum = 0;
            if (position == SWPNEXT)
	       bthdr->ofst += bthdr->blksz;
	    else
	       bthdr->ofst = ROOT;
	    if (bthdr->blk->count < 1)
	       {
	       bthdr->rcdnum = bthdr->blk->count + 1;
	       continue;
	       } /* if empty leaf record */
            } /* if good read */
         else if (!rdlen)
	    return(SWPEOF);
         else
            {
	    fprintf(stderr,"btsweep: %s: read error\n",
	       bthdr->dbname);
	    fprintf(stderr,"btsweep: read length %d\n",
	       rdlen);
            bthdr->reason = BTREE_ERR + 2302;
            return(BTREE_ERR + 2302);
            } /* if read length err */
         } /* if first read or end of block */
      while (bthdr->rcdnum < bthdr->blk->count)
	 {
         bthdr->rcd = (unsigned char *) bthdr->blk
            + bthdr->blkhdr
            + (bthdr->rcdnode * bthdr->rcdnum)
            + bthdr->rcdhdr;
         bthdr->key = (unsigned char *) bthdr->rcd
	    + bthdr->keyofst;
         bthdr->dletbyte = (unsigned char *) bthdr->rcd
	    + bthdr->dletofst;
         bthdr->rcdnum++;
         if (*bthdr->dletbyte != DELETED) break;
         } /* record loop within block */
      if (*bthdr->dletbyte != DELETED) break;
      } /* read loop */
   bthdr->seq++;
   if (buf != NULL)
      {
      p = (unsigned char *) bthdr->rcd;
      q = (unsigned char *) p + bthdr->rcdlen;
      r = (unsigned char *) buf;
      while (p < q) *r++ = *p++;
      } /* if buffer specified */
   return(SWPFOUND);
   } /* btsweep */
